@@ -36,7 +36,7 @@ int git_push_new(git_push **out, git_remote *remote)
 	*out = NULL;
 
 	p = git__calloc(1, sizeof(*p));
-	GITERR_CHECK_ALLOC(p);
+	GIT_ERROR_CHECK_ALLOC(p);
 
 	p->repo = remote->repo;
 	p->remote = remote;
@@ -70,10 +70,11 @@ int git_push_set_options(git_push *push, const git_push_options *opts)
 	if (!push || !opts)
 		return -1;
 
-	GITERR_CHECK_VERSION(opts, GIT_PUSH_OPTIONS_VERSION, "git_push_options");
+	GIT_ERROR_CHECK_VERSION(opts, GIT_PUSH_OPTIONS_VERSION, "git_push_options");
 
 	push->pb_parallelism = opts->pb_parallelism;
-	push->custom_headers = &opts->custom_headers;
+	push->connection.custom_headers = &opts->custom_headers;
+	push->connection.proxy = &opts->proxy_opts;
 
 	return 0;
 }
@@ -83,14 +84,14 @@ static void free_refspec(push_spec *spec)
 	if (spec == NULL)
 		return;
 
-	git_refspec__free(&spec->refspec);
+	git_refspec__dispose(&spec->refspec);
 	git__free(spec);
 }
 
 static int check_rref(char *ref)
 {
 	if (git__prefixcmp(ref, "refs/")) {
-		giterr_set(GITERR_INVALID, "not a valid reference '%s'", ref);
+		git_error_set(GIT_ERROR_INVALID, "not a valid reference '%s'", ref);
 		return -1;
 	}
 
@@ -108,10 +109,10 @@ static int check_lref(git_push *push, char *ref)
 		return 0;
 
 	if (error == GIT_ENOTFOUND)
-		giterr_set(GITERR_REFERENCE,
+		git_error_set(GIT_ERROR_REFERENCE,
 			"src refspec '%s' does not match any existing object", ref);
 	else
-		giterr_set(GITERR_INVALID, "not a valid reference '%s'", ref);
+		git_error_set(GIT_ERROR_INVALID, "not a valid reference '%s'", ref);
 	return -1;
 }
 
@@ -122,10 +123,10 @@ static int parse_refspec(git_push *push, push_spec **spec, const char *str)
 	*spec = NULL;
 
 	s = git__calloc(1, sizeof(*s));
-	GITERR_CHECK_ALLOC(s);
+	GIT_ERROR_CHECK_ALLOC(s);
 
 	if (git_refspec__parse(&s->refspec, str, false) < 0) {
-		giterr_set(GITERR_INVALID, "invalid refspec %s", str);
+		git_error_set(GIT_ERROR_INVALID, "invalid refspec %s", str);
 		goto on_error;
 	}
 
@@ -212,7 +213,7 @@ int git_push_update_tips(git_push *push, const git_remote_callbacks *callbacks)
 			if (error != GIT_ENOTFOUND)
 				goto on_error;
 
-			giterr_clear();
+			git_error_clear();
 			fire_callback = 0;
 		}
 
@@ -228,7 +229,7 @@ int git_push_update_tips(git_push *push, const git_remote_callbacks *callbacks)
 	error = 0;
 
 on_error:
-	git_buf_free(&remote_ref_name);
+	git_buf_dispose(&remote_ref_name);
 	return error;
 }
 
@@ -241,10 +242,10 @@ static int enqueue_tag(git_object **out, git_push *push, git_oid *id)
 	git_object *obj = NULL, *target = NULL;
 	int error;
 
-	if ((error = git_object_lookup(&obj, push->repo, id, GIT_OBJ_TAG)) < 0)
+	if ((error = git_object_lookup(&obj, push->repo, id, GIT_OBJECT_TAG)) < 0)
 		return error;
 
-	while (git_object_type(obj) == GIT_OBJ_TAG) {
+	while (git_object_type(obj) == GIT_OBJECT_TAG) {
 		if ((error = git_packbuilder_insert(push->pb, git_object_id(obj), NULL)) < 0)
 			break;
 
@@ -263,12 +264,11 @@ static int enqueue_tag(git_object **out, git_push *push, git_oid *id)
 	return error;
 }
 
-static int revwalk(git_vector *commits, git_push *push)
+static int queue_objects(git_push *push)
 {
 	git_remote_head *head;
 	push_spec *spec;
 	git_revwalk *rw;
-	git_oid oid;
 	unsigned int i;
 	int error = -1;
 
@@ -278,7 +278,7 @@ static int revwalk(git_vector *commits, git_push *push)
 	git_revwalk_sorting(rw, GIT_SORT_TIME);
 
 	git_vector_foreach(&push->specs, i, spec) {
-		git_otype type;
+		git_object_t type;
 		size_t size;
 
 		if (git_oid_iszero(&spec->loid))
@@ -294,13 +294,13 @@ static int revwalk(git_vector *commits, git_push *push)
 		if (git_odb_read_header(&size, &type, push->repo->_odb, &spec->loid) < 0)
 			goto on_error;
 
-		if (type == GIT_OBJ_TAG) {
+		if (type == GIT_OBJECT_TAG) {
 			git_object *target;
 
 			if ((error = enqueue_tag(&target, push, &spec->loid)) < 0)
 				goto on_error;
 
-			if (git_object_type(target) == GIT_OBJ_COMMIT) {
+			if (git_object_type(target) == GIT_OBJECT_COMMIT) {
 				if (git_revwalk_push(rw, git_object_id(target)) < 0) {
 					git_object_free(target);
 					goto on_error;
@@ -323,7 +323,7 @@ static int revwalk(git_vector *commits, git_push *push)
 				continue;
 
 			if (!git_odb_exists(push->repo->_odb, &spec->roid)) {
-				giterr_set(GITERR_REFERENCE, 
+				git_error_set(GIT_ERROR_REFERENCE,
 					"cannot push because a reference that you are trying to update on the remote contains commits that are not present locally.");
 				error = GIT_ENONFASTFORWARD;
 				goto on_error;
@@ -334,7 +334,7 @@ static int revwalk(git_vector *commits, git_push *push)
 
 			if (error == GIT_ENOTFOUND ||
 				(!error && !git_oid_equal(&base, &spec->roid))) {
-				giterr_set(GITERR_REFERENCE,
+				git_error_set(GIT_ERROR_REFERENCE,
 					"cannot push non-fastforwardable reference");
 				error = GIT_ENONFASTFORWARD;
 				goto on_error;
@@ -353,189 +353,23 @@ static int revwalk(git_vector *commits, git_push *push)
 		git_revwalk_hide(rw, &head->oid);
 	}
 
-	while ((error = git_revwalk_next(&oid, rw)) == 0) {
-		git_oid *o = git__malloc(GIT_OID_RAWSZ);
-		if (!o) {
-			error = -1;
-			goto on_error;
-		}
-		git_oid_cpy(o, &oid);
-		if ((error = git_vector_insert(commits, o)) < 0)
-			goto on_error;
-	}
+	error = git_packbuilder_insert_walk(push->pb, rw);
 
 on_error:
 	git_revwalk_free(rw);
-	return error == GIT_ITEROVER ? 0 : error;
-}
-
-static int enqueue_object(
-	const git_tree_entry *entry,
-	git_packbuilder *pb)
-{
-	switch (git_tree_entry_type(entry)) {
-		case GIT_OBJ_COMMIT:
-			return 0;
-		case GIT_OBJ_TREE:
-			return git_packbuilder_insert_tree(pb, entry->oid);
-		default:
-			return git_packbuilder_insert(pb, entry->oid, entry->filename);
-	}
-}
-
-static int queue_differences(
-	git_tree *base,
-	git_tree *delta,
-	git_packbuilder *pb)
-{
-	git_tree *b_child = NULL, *d_child = NULL;
-	size_t b_length = git_tree_entrycount(base);
-	size_t d_length = git_tree_entrycount(delta);
-	size_t i = 0, j = 0;
-	int error;
-
-	while (i < b_length && j < d_length) {
-		const git_tree_entry *b_entry = git_tree_entry_byindex(base, i);
-		const git_tree_entry *d_entry = git_tree_entry_byindex(delta, j);
-		int cmp = 0;
-
-		if (!git_oid__cmp(b_entry->oid, d_entry->oid))
-			goto loop;
-
-		cmp = strcmp(b_entry->filename, d_entry->filename);
-
-		/* If the entries are both trees and they have the same name but are
-		 * different, then we'll recurse after adding the right-hand entry */
-		if (!cmp &&
-			git_tree_entry__is_tree(b_entry) &&
-			git_tree_entry__is_tree(d_entry)) {
-			/* Add the right-hand entry */
-			if ((error = git_packbuilder_insert(pb, d_entry->oid,
-				d_entry->filename)) < 0)
-				goto on_error;
-
-			/* Acquire the subtrees and recurse */
-			if ((error = git_tree_lookup(&b_child,
-					git_tree_owner(base), b_entry->oid)) < 0 ||
-				(error = git_tree_lookup(&d_child,
-					git_tree_owner(delta), d_entry->oid)) < 0 ||
-				(error = queue_differences(b_child, d_child, pb)) < 0)
-				goto on_error;
-
-			git_tree_free(b_child); b_child = NULL;
-			git_tree_free(d_child); d_child = NULL;
-		}
-		/* If the object is new or different in the right-hand tree,
-		 * then enumerate it */
-		else if (cmp >= 0 &&
-			(error = enqueue_object(d_entry, pb)) < 0)
-			goto on_error;
-
-	loop:
-		if (cmp <= 0) i++;
-		if (cmp >= 0) j++;
-	}
-
-	/* Drain the right-hand tree of entries */
-	for (; j < d_length; j++)
-		if ((error = enqueue_object(git_tree_entry_byindex(delta, j), pb)) < 0)
-			goto on_error;
-
-	error = 0;
-
-on_error:
-	if (b_child)
-		git_tree_free(b_child);
-
-	if (d_child)
-		git_tree_free(d_child);
-
-	return error;
-}
-
-static int queue_objects(git_push *push)
-{
-	git_vector commits = GIT_VECTOR_INIT;
-	git_oid *oid;
-	size_t i;
-	unsigned j;
-	int error;
-
-	if ((error = revwalk(&commits, push)) < 0)
-		goto on_error;
-
-	git_vector_foreach(&commits, i, oid) {
-		git_commit *parent = NULL, *commit;
-		git_tree *tree = NULL, *ptree = NULL;
-		size_t parentcount;
-
-		if ((error = git_commit_lookup(&commit,	push->repo, oid)) < 0)
-			goto on_error;
-
-		/* Insert the commit */
-		if ((error = git_packbuilder_insert(push->pb, oid, NULL)) < 0)
-			goto loop_error;
-
-		parentcount = git_commit_parentcount(commit);
-
-		if (!parentcount) {
-			if ((error = git_packbuilder_insert_tree(push->pb,
-				git_commit_tree_id(commit))) < 0)
-				goto loop_error;
-		} else {
-			if ((error = git_tree_lookup(&tree, push->repo,
-					git_commit_tree_id(commit))) < 0 ||
-				(error = git_packbuilder_insert(push->pb,
-					git_commit_tree_id(commit), NULL)) < 0)
-				goto loop_error;
-
-			/* For each parent, add the items which are different */
-			for (j = 0; j < parentcount; j++) {
-				if ((error = git_commit_parent(&parent, commit, j)) < 0 ||
-					(error = git_commit_tree(&ptree, parent)) < 0 ||
-					(error = queue_differences(ptree, tree, push->pb)) < 0)
-					goto loop_error;
-
-				git_tree_free(ptree); ptree = NULL;
-				git_commit_free(parent); parent = NULL;
-			}
-		}
-
-		error = 0;
-
-	loop_error:
-		if (tree)
-			git_tree_free(tree);
-
-		if (ptree)
-			git_tree_free(ptree);
-
-		if (parent)
-			git_commit_free(parent);
-
-		git_commit_free(commit);
-
-		if (error < 0)
-			goto on_error;
-	}
-
-	error = 0;
-
-on_error:
-	git_vector_free_deep(&commits);
 	return error;
 }
 
 static int add_update(git_push *push, push_spec *spec)
 {
 	git_push_update *u = git__calloc(1, sizeof(git_push_update));
-	GITERR_CHECK_ALLOC(u);
+	GIT_ERROR_CHECK_ALLOC(u);
 
 	u->src_refname = git__strdup(spec->refspec.src);
-	GITERR_CHECK_ALLOC(u->src_refname);
+	GIT_ERROR_CHECK_ALLOC(u->src_refname);
 
 	u->dst_refname = git__strdup(spec->refspec.dst);
-	GITERR_CHECK_ALLOC(u->dst_refname);
+	GIT_ERROR_CHECK_ALLOC(u->dst_refname);
 
 	git_oid_cpy(&u->src, &spec->roid);
 	git_oid_cpy(&u->dst, &spec->loid);
@@ -556,7 +390,7 @@ static int calculate_work(git_push *push)
 			/* This is a create or update.  Local ref must exist. */
 			if (git_reference_name_to_id(
 					&spec->loid, push->repo, spec->refspec.src) < 0) {
-				giterr_set(GITERR_REFERENCE, "no such reference '%s'", spec->refspec.src);
+				git_error_set(GIT_ERROR_REFERENCE, "no such reference '%s'", spec->refspec.src);
 				return -1;
 			}
 		}
@@ -582,7 +416,7 @@ static int do_push(git_push *push, const git_remote_callbacks *callbacks)
 	git_transport *transport = push->remote->transport;
 
 	if (!transport->push) {
-		giterr_set(GITERR_NET, "remote transport doesn't support push");
+		git_error_set(GIT_ERROR_NET, "remote transport doesn't support push");
 		error = -1;
 		goto on_error;
 	}
@@ -642,7 +476,7 @@ int git_push_finish(git_push *push, const git_remote_callbacks *callbacks)
 	int error;
 
 	if (!git_remote_connected(push->remote) &&
-	    (error = git_remote_connect(push->remote, GIT_DIRECTION_PUSH, callbacks, NULL, push->custom_headers)) < 0)
+	    (error = git_remote__connect(push->remote, GIT_DIRECTION_PUSH, callbacks, &push->connection)) < 0)
 		return error;
 
 	if ((error = filter_refs(push->remote)) < 0 ||
@@ -651,7 +485,7 @@ int git_push_finish(git_push *push, const git_remote_callbacks *callbacks)
 
 	if (!push->unpack_ok) {
 		error = -1;
-		giterr_set(GITERR_NET, "unpacking the sent packfile failed on the remote");
+		git_error_set(GIT_ERROR_NET, "unpacking the sent packfile failed on the remote");
 	}
 
 	return error;
@@ -667,7 +501,7 @@ int git_push_status_foreach(git_push *push,
 	git_vector_foreach(&push->status, i, status) {
 		int error = cb(status->ref, status->msg, data);
 		if (error)
-			return giterr_set_after_callback(error);
+			return git_error_set_after_callback(error);
 	}
 
 	return 0;
